package Activator::Tutorial;

__END__

=head1 NAME

Activator Development Framework Tutorial


=head1 INTRODUCTION

1. When working on a Catalyst web abpp, dev is very different than QA than Production, but most articles, documentation and modules assume everything done from Catalyst
2. There is no better design pattern than MVC for web apps
3. Catalyst is great, Activator tries to fill in some missing parts

=head1 Getting Started: Creating a Catalyst/Activator project

=head1 Databases: ORM or Write SQL? Pagination too!

If you want to always write your own SQL (and sometimes you do), Activator::DB is for you.

=head1 Logging: put it in the right place

=head1 Exceptions: handle program errors with destination aware messaging

=head1 Beyond Catalyst: writing crons and CLI backend apps

=head1 Email: talk to the rest of the world simply

=head1 Appendix A: AJAX Framework


=head1 AUTHOR

Karim A. Nassar

=head1 COPYRIGHT

Copyright (c) 2007 Karim A. Nassar <karim.nassar@acm.org>

You may distribute under the terms of either the GNU General Public
License or the Artistic License, as specified in the Perl README file.

=cut
